@@ -17,14 +17,15 @@ from urlparse import parse_qs
 from datetime import datetime
 from traceback import print_exc
 from sys import stderr, stdout, exc_info
+from types import GeneratorType
 try:
 	from ujson import encode as json_dumps
 except ImportError:
 	from json import dumps as json_dumps
 
-#from gevent.pywsgi import WSGIServer as gWSGIServer
-from gevent.wsgi import WSGIServer as gWSGIServer
-from gevent.pool import Pool
+from gevent.pywsgi import WSGIServer as pywsgi_server
+from gevent.wsgi import WSGIServer as wsgi_server
+#from gevent.pool import Pool
 
 from content_types import *
 
@@ -33,7 +34,7 @@ __author__		= "Justin Wilson"
 __copyright__	= "Copyright 2011, Justin Wilson"
 __credits__		= ["Justin Wilson", "Eber Irigoyen", "Corey McKinnon"]
 __license__		= "Simplified BSD License"
-__version__		= "0.1.1"
+__version__		= "0.1.2"
 __maintainer__	= "Justin Wilson"
 __email__		= "justinwilson1 at gmail dot com"
 __status__		= "Alpha"
@@ -41,6 +42,29 @@ __status__		= "Alpha"
 status_codes = BaseHTTPRequestHandler.responses
 
 '''________________________________helpers___________________________________'''
+def _body_wrap(content_type):
+	def normal(data):
+		return data
+	def json(data):
+		j_data, e = get_json(data)
+		if e: log_error(e)
+		return j_data
+
+	if content_type is APPLICATION.JSON:
+		return json
+	else:
+		return normal
+
+def _poll(body, content_type):
+	fn = _body_wrap(content_type)
+	for b in body:
+		try:
+			yield fn(b)
+		except:
+			err = exc_info()[1]
+			log_error(err)
+			yield ''
+
 def log_error(err):
 	'''Logs the given exception, and it's corresponding stacktrace'''
 	# TODO(jwilson): Implement logging to output to a real location
@@ -71,20 +95,17 @@ def parse_vals(env, *args):
 
 	return tuple([data.get(key, default)[0] for key in args])
 
-def get_json(body, callback):
+def get_json(body, callback=None):
 	''' Converts the `body` to JSON, except when `body` is a str and returns the
 		result, wrapping it in the callback when provided.
 	'''
 	result, err = '', None
 
 	try:
-		if isinstance(body, dict):
-			result = json_dumps(body)
-		elif isinstance(body, str):
+		if isinstance(body, str):
 			result = body
 		else:
-			b_type = type(body)
-			raise TypeError("JSON result must be 'dict' or 'str', received: '{0}'".format(b_type))
+			result = json_dumps(body)
 	except TypeError, e:
 		err = e
 	else:
@@ -93,13 +114,13 @@ def get_json(body, callback):
 
 	return result, err
 
-def error_handler(env, resp, code, exception=None, content_type=TEXT.HTML):
+def error_handler(env, resp, code, err=None, content_type=TEXT.HTML):
 	''' Handler used to log the given exception, if given, and return
 		an error message to be sent to the response.
 	'''
 
-	if exception is not None:
-		log_error(exception)
+	if err is not None:
+		log_error(err)
 
 	resp(status(code), content_type)
 
@@ -109,71 +130,82 @@ def error_handler(env, resp, code, exception=None, content_type=TEXT.HTML):
 		return 'error: {0}'.format(code)
 
 def handle_route(env, resp, route, kwargs):
-	body = None										# response body
+	''' Handler used to process routes.
+		Supports: Yielding (polling) of routes
+				  JSON dumping when `Content-Type` is JSON
+				  gevent Queues
+	'''
+
 	query_data = parse_qs(env.get('QUERY_STRING'))	# query string
 	callback = query_data.pop('callback', None)		# callback values
 	env['qs'] = query_data
 
-	#if route.method == 'POST':
 	if env.get('REQUEST_METHOD') == 'POST':
 		form_vals = env['wsgi.input'].read()
 		env['post_data'] = parse_qs(form_vals)
-
+	
 	try:
-		if kwargs:
-			body = route(env, resp, **kwargs)
-		else:
-			body = route(env, resp)
+		body = route(env, resp, **kwargs)
 	except:
 		e = exc_info()[1]
 		return error_handler(env, resp, 500, e, route.content_type)
+	else:
+		if isinstance(body, GeneratorType):
+			body = _poll(body, route.content_type)	# get generator
+		elif route.content_type is APPLICATION.JSON:
+			body, err = get_json(body, callback)	# get content
+			if err:									# error immediatly
+				return error_handler(env, resp, 500, err, route.content_type)
 
-	if route.content_type == APPLICATION.JSON:
-		body, err = get_json(body, callback)
-		if err:
-			return error_handler(env, resp, 500, err, APPLICATION.JSON)
-
-	resp(status(200), route.content_type)
-	return body
-
-def _print_start_msg(addr, port):
-	print('[gserver] {0} - listening on {1}:{2}'
-			.format(datetime.today(), addr, port))
+		resp(status(200), route.content_type)		# success
+		return body
 
 '''________________________________server____________________________________'''
 class WSGIServer:
-	def __init__(self, addr, port, routes, **kwargs):
+	def __init__(self, addr, routes, **kwargs):
+		""" Creates a new WSGIServer using either the gevent underlying wsgi/pywsgi.
+			
+			When passing in the argument `enable_streaming` the gevent pywsgi
+			server is created, to enable polling. Otherwise the faster
+			libevent-http is used.
+			(*libevent does not support SSL, streaming or pipelining*)
+		"""
 		self._routes = routes
-		self._port = port
-		self._addr = addr if addr else '127.0.0.1'
-		self._server = gWSGIServer( (addr, port),
-									    self._app_handler,
-									    **kwargs )
+		self._port = addr[1]
+		self._addr = addr[0] if addr[0] else '127.0.0.1'
+
+		if kwargs.pop('enable_streaming', False):
+			wsgi = pywsgi_server
+		else:
+			wsgi = wsgi_server
+
+		self._server = wsgi(addr, self._handler, **kwargs)
 	
 	def _print_start_msg(self):
 		print('[gserver] {0} - listening on {1}:{2}'
 				.format(datetime.today(), self._addr, self._port))
 
 	def start(self):
-		_print_start_msg(self._addr, self._port)
+		self._print_start_msg()
 		self._server.start()
 
 	def stop(self):
-		_print_start_msg(self._addr, self._port)
+		self._print_start_msg()
 		self._server.stop()
 
 	def serve_forever(self):
-		_print_start_msg(self._addr, self._port)
+		self._print_start_msg()
 		self._server.serve_forever()
-	
-	def _app_handler(self, env, resp):
+
+	def _handler(self, env, resp):
 		path = env.get('PATH_INFO')
 		method = env.get('REQUEST_METHOD')
 		route, kwargs = self._routes.get(path)
 
 		if not route:
-			yield error_handler(env, resp, 404)
+			return error_handler(env, resp, 404)
 		elif method != route.method and method not in route.methods:
-			yield error_handler(env, resp, 501, content_type=route.content_type)
+			return error_handler(env, resp, 501, content_type=route.content_type)
 		else:
-			yield handle_route(env, resp, route, kwargs)
+			return handle_route(env, resp, route, kwargs)
+
